@@ -1,8 +1,9 @@
 import os
 from pathlib import Path
 from typing import List
+import pkg_resources
 
-from tree_sitter import Node, Language, Parser, Tree
+from tree_sitter import Node, Language, Parser, Tree, TreeCursor
 
 from tree_hugger.exceptions import ParserLibraryNotFoundError, SourceFileNotFoundError, QueryFileNotFoundError
 from tree_hugger.core.queries import Query
@@ -24,10 +25,8 @@ class BaseParser(object):
     """
     Base parser exposes the common interface that we extend per language
     """
-    
-    DEFAULT_QUERY_FILE_PATH = str(Path("tree_hugger") / "assets" / "queries" / "queries.yml")
 
-    def __init__(self, language: str, query_class_name: str, library_loc: str=None, query_file_path: str=None):
+    def __init__(self, language: str, query_class_name: str, query_file_path: str, library_loc: str=None):
         if os.getenv("TS_LIB_PATH") is not None and library_loc is None:
             library_loc = os.getenv("TS_LIB_PATH")
         
@@ -37,17 +36,10 @@ class BaseParser(object):
         if not Path(library_loc).exists() or not Path(library_loc).is_file():
             raise ParserLibraryNotFoundError(f"Parser library '{library_loc}' not found. Did you set up the environement variables?")
         
-        if os.getenv("QUERY_FILE_PATH") is not None and query_file_path is None:
-            query_file_path =  os.getenv("QUERY_FILE_PATH")
-        
-        if not query_file_path:
-            query_file_path = BaseParser.DEFAULT_QUERY_FILE_PATH
-
-        
         self.language = Language(library_loc, language)
         self.parser = Parser()
         self.parser.set_language(self.language)
-        self.qclass = Query(query_file_path)
+        self.qclass = Query.fromFile(query_file_path)
         self.QUERIES = self.qclass[query_class_name]
     
     def _run_query_and_get_captures(self, q_name: str, root_node:  Node) -> List:
@@ -81,19 +73,38 @@ class BaseParser(object):
     def sexp(self):
         return self.tree.root_node.sexp()
     
-    def reload_queries(self, query_file_path: str=None):
+    def reload_queries(self):
         """
         Reloads the query file and the internal data structure.
-
+    
         This is a temporary method and it should not be a part of the API. 
         mainly to dynamically reload the queries while in development for 
         IPython console
         """
-        if os.getenv("QUERY_FILE_PATH") is not None and query_file_path is None:
-            query_file_path =  os.getenv("QUERY_FILE_PATH")
-        
-        self.qclass = Query(query_file_path)
-        self.QUERIES = self.qclass['python_queries']
-    
+        self.qclass.reload()
+
     def _has_children(self, node: Node) -> bool:
         return len(node.children) > 0
+        
+    def _walk_recursive(self, cursor: TreeCursor, apply):
+        apply(cursor.node)
+        for ch in cursor.node.children:
+            self._walk_recursive(ch.walk(), apply)
+    
+    def _reduce_recursive(self, cursor: TreeCursor, reduction, accumulator):
+        acc = accumulator
+        for ch in cursor.node.children:
+            acc = self._reduce_recursive(ch.walk(), reduction, reduction(ch, acc))
+        return acc
+
+    def walk(self, apply):
+        """
+        Iterate over the tree of the parsed code and apply the given 
+        """ 
+        self._walk_recursive(self.root_node.walk(), apply)
+    
+    def reduction(self, reduction, neutral):
+        """
+        Apply a reduction operation over the tree of the parsed code
+        """ 
+        return self._reduce_recursive(self.root_node.walk(), reduction, reduction(self.root_node, neutral))
